<?php
/**
 * 郵便番号・住所サジェスト システム
 *
 * @version   1.00.00
 * @copyright (c) 2019 Step One co,.ltd.
 * @license   Commercial License
 * @see       https://www.steponecoltd.com/product/p2as/
 */
namespace StepOne\PostcodeToAddressSuggester;

const DB_FILEPATH = './data/p2as_NWJiODVl_{type}.db';
const LICENSE_NO  = '8888-8888-8888-8888-8871';


class Search
{
    protected $postcode;
    protected $type;
    protected $db;

     /**
     * コンストラクタ
     */
    public function __construct($postcode, $type)
    {
        $this->postcode = $postcode;
        $this->type     = $type;

        $this->initialize();

        $val = $this->search();

        $this->output($val);

        $this->finalize();
    }

    /**
     * 初期化
     */
    protected function initialize()
    {
        $type = $this->type;
        if ($type !== '1to1' && $type !== 'jigyosyo' && $type !== 'multi') {
            $this->error('type query is invalid.');
        }

        //クエリーのチェック
        $postcode = $this->postcode;
        if (!$postcode) {
            $this->error('postcode query is empty.');
        } elseif (!ctype_digit($postcode)) {
            $this->error('postcode query is invalid.');
        }

        //データベース ファイルを探す
        $dbType = [ $type, 'multi', 'jigyosyo' ];
        $dbFile = null;
        foreach ($dbType as $t) {
            $t = str_replace('{type}', $t, DB_FILEPATH);
            if (is_file($t)) {
              $dbFile = $t;
            }
        }
        if (!$dbFile) {
            $this->error('data file not exist.');
        }

        //データベースを開く
        $this->db = $db = new \SQLite3($dbFile);
        $db->exec('PRAGMA journal_mode = WAL;');
    }

    /**
     * 終了化
     */
    protected function finalize()
    {
        //データベースを閉じる
        if ($this->db) {
            $this->db->close();
            $this->db = null;
        }
   }

    /**
     * エラー
     */
    protected function error($mes)
    {
        print '[ PostcodeToAddressSuggester ERROR ] ' . $mes;

        //データベースを開いている場合は閉じる
        if ($this->db) {
            $this->finalize();
        }

        exit;
    }

    /**
     * 検索
     */
    protected function search()
    {
        $postcode = $this->postcode;

        //検索
        $sql = 'SELECT data FROM postdata WHERE code = ' . $postcode . ' LIMIT 1;';
        $res = $this->db->query($sql);
        $row = $res->fetchArray(\SQLITE3_NUM);

        if (!$row) {
            $this->error('data no match.');
        }

        return $row[0];
    }

    /**
     * 出力
     */
    protected function output($val)
    {
        //ヘッダーを出力
        header('X-Software: PostcodeToAddressSuggester');
        header('X-Vendor: Step One co,.ltd. ');
        if (ord($val) === 31) {   //圧縮版
            header('Content-Encoding: gzip');
            header('Vary: Accept-Encoding');
        }

        //出力
        print $val;
    }
}
new Search((isset($_GET['postcode']) ? $_GET['postcode'] : ''), $_GET['type']);
